@@ -44,7 +44,6 @@
 #include "egg-string.h"
 #include "egg-dbus-monitor.h"
 
-#include "gpk-consolekit.h"
 #include "gpk-common.h"
 #include "gpk-gnome.h"
 #include "gpk-auto-refresh.h"
@@ -60,6 +59,9 @@ static void     gpk_check_update_finalize	(GObject	     *object);
 /* the maximum number of lines of data on the libnotify widget */
 #define GPK_CHECK_UPDATE_MAX_NUMBER_SECURITY_ENTRIES	7
 
+/* the amount of time after ::UpdatesChanged we refresh the update list */
+#define GPK_CHECK_UPDATE_UPDATES_CHANGED_TIMEOUT	3 /* seconds */
+
 struct GpkCheckUpdatePrivate
 {
 	GtkStatusIcon		*status_icon;
@@ -78,6 +80,7 @@ struct GpkCheckUpdatePrivate
 	NotifyNotification	*notification_updates_available;
 	GPtrArray		*important_updates_array;
 	EggDbusMonitor		*dbus_monitor_viewer;
+	guint			 updates_changed_id;
 };
 
 G_DEFINE_TYPE (GpkCheckUpdate, gpk_check_update, G_TYPE_OBJECT)
@@ -222,17 +225,17 @@ gpk_check_update_show_about_cb (GtkMenuItem *item, gpointer data)
 		NULL};
 	const char *license[] = {
 		N_("Licensed under the GNU General Public License Version 2"),
-		N_("PackageKit is free software; you can redistribute it and/or\n"
-		   "modify it under the terms of the GNU General Public License\n"
-		   "as published by the Free Software Foundation; either version 2\n"
+		N_("PackageKit is free software; you can redistribute it and/or "
+		   "modify it under the terms of the GNU General Public License "
+		   "as published by the Free Software Foundation; either version 2 "
 		   "of the License, or (at your option) any later version."),
-		N_("PackageKit is distributed in the hope that it will be useful,\n"
-		   "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+		N_("PackageKit is distributed in the hope that it will be useful, "
+		   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
 		   "GNU General Public License for more details."),
-		N_("You should have received a copy of the GNU General Public License\n"
-		   "along with this program; if not, write to the Free Software\n"
-		   "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA\n"
+		N_("You should have received a copy of the GNU General Public License "
+		   "along with this program; if not, write to the Free Software "
+		   "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA "
 		   "02110-1301, USA.")
 	};
 	const char *translators = _("translator-credits");
@@ -253,6 +256,7 @@ gpk_check_update_show_about_cb (GtkMenuItem *item, gpointer data)
 			       "version", VERSION,
 			       "copyright", "Copyright \xc2\xa9 2007-2009 Richard Hughes",
 			       "license", license_trans,
+			       "wrap-license", TRUE,
 				/* TRANSLATORS: website label */
 			       "website-label", _("PackageKit Website"),
 			       "website", "www.packagekit.org",
@@ -366,64 +370,17 @@ out:
 	return ret;
 }
 
-/**
- * gpk_check_update_menuitem_update_system_cb:
- **/
-static void
-gpk_check_update_menuitem_update_system_cb (GtkMenuItem *item, gpointer data)
-{
-	GpkCheckUpdate *cupdate = GPK_CHECK_UPDATE (data);
-	g_return_if_fail (GPK_IS_CHECK_UPDATE (cupdate));
-	gpk_check_update_update_system (cupdate);
-}
-
-/**
- * gpk_check_update_menuitem_show_updates_cb:
- **/
-static void
-gpk_check_update_menuitem_show_updates_cb (GtkMenuItem *item, gpointer data)
-{
-	const gchar *command = "gpk-update-viewer";
-	if (!g_spawn_command_line_async (command, NULL))
-		egg_warning ("Couldn't execute command: %s", command);
-}
-
 /**
  * gpk_check_update_activate_update_cb:
- * @button: Which buttons are pressed
  *
- * Callback when the icon is clicked
+ * Callback when the icon is left clicked
  **/
 static void
 gpk_check_update_activate_update_cb (GtkStatusIcon *status_icon, GpkCheckUpdate *icon)
 {
-	GtkMenu *menu = (GtkMenu*) gtk_menu_new ();
-	GtkWidget *item;
-	GtkWidget *image;
-
-	egg_debug ("icon left clicked");
-
-	/* TRANSLATORS: context menu to execute the update viewer  */
-	item = gtk_image_menu_item_new_with_mnemonic (_("_Show Updates"));
-	image = gtk_image_new_from_icon_name (GPK_ICON_SOFTWARE_UPDATE, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-	g_signal_connect (G_OBJECT (item), "activate",
-			  G_CALLBACK (gpk_check_update_menuitem_show_updates_cb), icon);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-
-	/* TRANSLATORS: context menu to update any pending updates */
-	item = gtk_image_menu_item_new_with_mnemonic (_("_Update System Now"));
-	image = gtk_image_new_from_icon_name (GPK_ICON_SOFTWARE_UPDATE_AVAILABLE, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-	g_signal_connect (G_OBJECT (item), "activate",
-			  G_CALLBACK (gpk_check_update_menuitem_update_system_cb), icon);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-
-	/* show the menu */
-	gtk_widget_show_all (GTK_WIDGET (menu));
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-			gtk_status_icon_position_menu, status_icon,
-			1, gtk_get_current_event_time());
+	const gchar *command = "gpk-update-viewer";
+	if (!g_spawn_command_line_async (command, NULL))
+		egg_warning ("Couldn't execute command: %s", command);
 }
 
 /**
@@ -437,37 +394,43 @@ gpk_check_update_libnotify_cb (NotifyNotification *notification, gchar *action,
 	gchar **package_ids;
 	GpkCheckUpdate *cupdate = GPK_CHECK_UPDATE (data);
 
-	if (egg_strequal (action, "do-not-show-complete-restart")) {
+	if (g_strcmp0 (action, "do-not-show-complete-restart") == 0) {
 		egg_debug ("set %s to FALSE", GPK_CONF_NOTIFY_UPDATE_COMPLETE_RESTART);
 		gconf_client_set_bool (cupdate->priv->gconf_client, GPK_CONF_NOTIFY_UPDATE_COMPLETE_RESTART, FALSE, NULL);
-	} else if (egg_strequal (action, "do-not-show-complete")) {
+	} else if (g_strcmp0 (action, "do-not-show-complete") == 0) {
 		egg_debug ("set %s to FALSE", GPK_CONF_NOTIFY_UPDATE_COMPLETE);
 		gconf_client_set_bool (cupdate->priv->gconf_client, GPK_CONF_NOTIFY_UPDATE_COMPLETE, FALSE, NULL);
-	} else if (egg_strequal (action, "do-not-show-update-started")) {
+	} else if (g_strcmp0 (action, "do-not-show-update-started") == 0) {
 		egg_debug ("set %s to FALSE", GPK_CONF_NOTIFY_UPDATE_STARTED);
 		gconf_client_set_bool (cupdate->priv->gconf_client, GPK_CONF_NOTIFY_UPDATE_STARTED, FALSE, NULL);
-	} else if (egg_strequal (action, "do-not-show-notify-critical")) {
+	} else if (g_strcmp0 (action, "do-not-show-notify-critical") == 0) {
 		egg_debug ("set %s to FALSE", GPK_CONF_NOTIFY_CRITICAL);
 		gconf_client_set_bool (cupdate->priv->gconf_client, GPK_CONF_NOTIFY_CRITICAL, FALSE, NULL);
-	} else if (egg_strequal (action, "do-not-show-update-not-battery")) {
+	} else if (g_strcmp0 (action, "do-not-show-update-not-battery") == 0) {
 		egg_debug ("set %s to FALSE", GPK_CONF_NOTIFY_UPDATE_NOT_BATTERY);
 		gconf_client_set_bool (cupdate->priv->gconf_client, GPK_CONF_NOTIFY_UPDATE_NOT_BATTERY, FALSE, NULL);
-	} else if (egg_strequal (action, "distro-upgrade-do-not-show-available")) {
+	} else if (g_strcmp0 (action, "distro-upgrade-do-not-show-available") == 0) {
 		egg_debug ("set %s to FALSE", GPK_CONF_NOTIFY_DISTRO_UPGRADES);
 		gconf_client_set_bool (cupdate->priv->gconf_client, GPK_CONF_NOTIFY_DISTRO_UPGRADES, FALSE, NULL);
 //	} else if (egg_strequal (action, "show-error-details")) {
 //		/* TRANSLATORS: detailed text about the error */
 //		gpk_error_dialog (_("Error details"), _("Package Manager error details"), cupdate->priv->error_details);
-	} else if (egg_strequal (action, "cancel")) {
+	} else if (g_strcmp0 (action, "cancel") == 0) {
 		/* try to cancel */
 		ret = pk_client_cancel (cupdate->priv->client_primary, &error);
 		if (!ret) {
 			egg_warning ("failed to cancel client: %s", error->message);
 			g_error_free (error);
 		}
-	} else if (egg_strequal (action, "update-all-packages")) {
+	} else if (g_strcmp0 (action, "update-all-packages") == 0) {
 		gpk_check_update_update_system (cupdate);
-	} else if (egg_strequal (action, "update-just-security")) {
+	} else if (g_strcmp0 (action, "show-update-viewer") == 0) {
+		ret = g_spawn_command_line_async (BINDIR "/gpk-update-viewer", &error);
+		if (!ret) {
+			egg_warning ("Failure launching update viewer: %s", error->message);
+			g_error_free (error);
+		}
+	} else if (g_strcmp0 (action, "update-just-security") == 0) {
 
 		ret = pk_client_reset (cupdate->priv->client_primary, &error);
 		if (!ret) {
@@ -489,10 +452,11 @@ gpk_check_update_libnotify_cb (NotifyNotification *notification, gchar *action,
 		}
 		g_strfreev (package_ids);
 
-	} else if (egg_strequal (action, "distro-upgrade-info")) {
-		ret = g_spawn_command_line_async ("/usr/share/PackageKit/pk-upgrade-distro.sh", NULL);
+	} else if (g_strcmp0 (action, "distro-upgrade-info") == 0) {
+		ret = g_spawn_command_line_async (DATADIR "/PackageKit/pk-upgrade-distro.sh", &error);
 		if (!ret) {
-			egg_warning ("Failure launching pk-upgrade-distro.sh");
+			egg_warning ("Failure launching pk-upgrade-distro.sh: %s", error->message);
+			g_error_free (error);
 		}
 	} else {
 		egg_warning ("unknown action id: %s", action);
@@ -566,7 +530,7 @@ gpk_check_update_critical_updates_warning (GpkCheckUpdate *cupdate, const gchar
 
 	/* do the bubble */
 	egg_debug ("title=%s, message=%s", title, message);
-	notification = notify_notification_new (title, message, "help-browser", NULL);
+	notification = notify_notification_new_with_status_icon (title, message, "help-browser", cupdate->priv->status_icon);
 	if (notification == NULL) {
 		egg_warning ("failed to get bubble");
 		return;
@@ -576,12 +540,15 @@ gpk_check_update_critical_updates_warning (GpkCheckUpdate *cupdate, const gchar
 	notify_notification_add_action (notification, "update-just-security",
 					/* TRANSLATORS: button: only security updates */
 					_("Install only security updates"), gpk_check_update_libnotify_cb, cupdate, NULL);
-	notify_notification_add_action (notification, "update-all-packages",
-					/* TRANSLATORS: button: all pending updates */
-					_("Install all updates"), gpk_check_update_libnotify_cb, cupdate, NULL);
-	notify_notification_add_action (notification, "do-not-show-notify-critical",
-					/* TRANSLATORS: button: hide forever */
-					_("Do not show this again"), gpk_check_update_libnotify_cb, cupdate, NULL);
+//	notify_notification_add_action (notification, "update-all-packages",
+//					/* TRANSLATORS: button: all pending updates */
+//					_("Install all updates"), gpk_check_update_libnotify_cb, cupdate, NULL);
+	notify_notification_add_action (notification, "show-update-viewer",
+					/* TRANSLATORS: button: open the update viewer */
+					_("Show all software updates"), gpk_check_update_libnotify_cb, cupdate, NULL);
+//	notify_notification_add_action (notification, "do-not-show-notify-critical",
+//					/* TRANSLATORS: button: hide forever */
+//					_("Do not show this again"), gpk_check_update_libnotify_cb, cupdate, NULL);
 	ret = notify_notification_show (notification, &error);
 	if (!ret) {
 		egg_warning ("error: %s", error->message);
@@ -692,12 +659,14 @@ gpk_check_update_check_on_battery (GpkCheckUpdate *cupdate)
 	/* TRANSLATORS: policy says update, but we are on battery and so prompt */
 	message = _("Automatic updates are not being installed as the computer is running on battery power");
 	/* TRANSLATORS: informs user will not install by default */
-	notification = notify_notification_new (_("Updates not installed"), message, "help-browser", NULL);
+	notification = notify_notification_new_with_status_icon (_("Updates not installed"),
+								 message, "help-browser",
+								 cupdate->priv->status_icon);
 	notify_notification_set_timeout (notification, 15000);
 	notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
-	notify_notification_add_action (notification, "do-not-show-update-not-battery",
-					/* TRANSLATORS: hide this warning type forever */
-					_("Do not show this warning again"), gpk_check_update_libnotify_cb, cupdate, NULL);
+//	notify_notification_add_action (notification, "do-not-show-update-not-battery",
+//					/* TRANSLATORS: hide this warning type forever */
+//					_("Do not show this warning again"), gpk_check_update_libnotify_cb, cupdate, NULL);
 	notify_notification_add_action (notification, "update-all-packages",
 					/* TRANSLATORS: to hell with my battery life, just do it */
 					_("Install the updates anyway"), gpk_check_update_libnotify_cb, cupdate, NULL);
@@ -782,18 +751,18 @@ gpk_check_update_notify_doing_updates (GpkCheckUpdate *cupdate)
 		goto out;
 
 	/* TRANSLATORS: title: notification when we scheduled an automatic update */
-	notification = notify_notification_new (_("Updates are being installed"),
+	notification = notify_notification_new_with_status_icon (_("Updates are being installed"),
 						/* TRANSLATORS: tell the user why the hard disk is grinding... */
 						_("Updates are being automatically installed on your computer"),
-						"software-update-urgent", NULL);
+						"software-update-urgent", cupdate->priv->status_icon);
 	notify_notification_set_timeout (notification, 15000);
 	notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
 	/* TRANSLATORS: button: cancel the update system */
 	notify_notification_add_action (notification, "cancel",
 					_("Cancel update"), gpk_check_update_libnotify_cb, cupdate, NULL);
 	/* TRANSLATORS: button: don't show this again */
-	notify_notification_add_action (notification, "do-not-show-update-started",
-					_("Do not show this again"), gpk_check_update_libnotify_cb, cupdate, NULL);
+//	notify_notification_add_action (notification, "do-not-show-update-started",
+//					_("Do not show this again"), gpk_check_update_libnotify_cb, cupdate, NULL);
 	ret = notify_notification_show (notification, &error);
 	if (!ret) {
 		egg_warning ("error: %s", error->message);
@@ -885,10 +854,12 @@ gpk_check_update_process_updates (GpkCheckUpdate *cupdate, PkPackageList *list,
 					obj->id->name, obj->summary);
 		if (++showing == GPK_CHECK_UPDATE_MAX_NUMBER_SECURITY_ENTRIES) {
 			more = security_array->len - showing;
-			/* TRANSLATORS: we have a notification that won't fit, so append on how many other we are not showing */
-			g_string_append_printf (status_security, ngettext ("and %d other security update",
-									   "and %d other security updates", more), more);
-			g_string_append (status_security, "...\n");
+			if (more > 0) {
+				/* TRANSLATORS: we have a notification that won't fit, so append on how many other we are not showing */
+				g_string_append_printf (status_security, ngettext ("and %d other security update",
+										   "and %d other security updates", more), more);
+				g_string_append (status_security, "...\n");
+			}
 			break;
 		}
 	}
@@ -942,7 +913,9 @@ gpk_check_update_process_updates (GpkCheckUpdate *cupdate, PkPackageList *list,
 
 	/* are we on battery and configured to skip the action */
 	ret = gpk_check_update_check_on_battery (cupdate);
-	if (!ret) {
+	if (!ret &&
+	    ((update == GPK_UPDATE_ENUM_SECURITY && security_array->len > 0) ||
+	      update == GPK_UPDATE_ENUM_ALL)) {
 		egg_debug ("on battery so not doing update");
 
 		/* play the sound, using sounds from the naming spec */
@@ -1019,6 +992,18 @@ gpk_check_update_query_updates_idle_cb (GpkCheckUpdate *cupdate)
 	return FALSE;
 }
 
+/**
+ * gpk_check_update_query_updates_changed_cb:
+ **/
+static gboolean
+gpk_check_update_query_updates_changed_cb (GpkCheckUpdate *cupdate)
+{
+	egg_debug ("getting new update list (after we waited a short delay)");
+	cupdate->priv->updates_changed_id = 0;
+	gpk_check_update_query_updates (cupdate, TRUE);
+	return FALSE;
+}
+
 /**
  * gpk_check_update_updates_changed_cb:
  **/
@@ -1036,9 +1021,15 @@ gpk_check_update_updates_changed_cb (PkControl *control, GpkCheckUpdate *cupdate
 		return;
 	}
 
+	/* if we get this in the timeout, just remove and start again from now */
+	if (cupdate->priv->updates_changed_id > 0)
+		g_source_remove (cupdate->priv->updates_changed_id);
+
 	/* now try to get newest update list */
-	egg_debug ("updates changed, so getting new update list");
-	g_idle_add ((GSourceFunc) gpk_check_update_query_updates_idle_cb, cupdate);
+	egg_debug ("updates changed, so getting new update list soon");
+	cupdate->priv->updates_changed_id =
+		g_timeout_add_seconds (GPK_CHECK_UPDATE_UPDATES_CHANGED_TIMEOUT,
+				       (GSourceFunc) gpk_check_update_query_updates_changed_cb, cupdate);
 }
 
 /**
@@ -1186,7 +1177,7 @@ gpk_check_update_process_distro_upgrades (GpkCheckUpdate *cupdate, PkObjList *ar
 
 	/* TRANSLATORS: a distro update is available, e.g. Fedora 8 to Fedora 9 */
 	title = _("Distribution upgrades available");
-	notification = notify_notification_new (title, string->str, "help-browser", NULL);
+	notification = notify_notification_new_with_status_icon (title, string->str, "help-browser", cupdate->priv->status_icon);
 	if (notification == NULL) {
 		egg_warning ("failed to get bubble");
 		return;
@@ -1239,22 +1230,46 @@ gpk_cupdate_connection_changed_cb (EggDbusMonitor *monitor, gboolean connected,
 static void
 gpk_check_update_error_code_cb (PkClient *client, PkErrorCodeEnum code, const gchar *details, GpkCheckUpdate *cupdate)
 {
+	PkRoleEnum role;
+	gboolean ret;
+	GError *error = NULL;
+
 	/* ignore some errors */
 	if (code == PK_ERROR_ENUM_PROCESS_KILL ||
 	    code == PK_ERROR_ENUM_TRANSACTION_CANCELLED) {
 		egg_debug ("error ignored %s\n%s", pk_error_enum_to_text (code), details);
-		return;
+		goto out;
 	}
 
 	/* ignore the ones we can handle */
 	if (pk_error_code_is_need_untrusted (code)) {
 		egg_debug ("error ignored as we're handling %s\n%s", pk_error_enum_to_text (code), details);
-		return;
+		goto out;
+	}
+
+	/* get the role */
+	ret = pk_client_get_role (client, &role, NULL, &error);
+	if (!ret) {
+		egg_warning ("failed to get role: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* if we're doing queries automatically for the user, don't spam them
+	   with locking failure messages */
+	if (code == PK_ERROR_ENUM_CANNOT_GET_LOCK) {
+		if (role == PK_ROLE_ENUM_GET_UPDATES ||
+		    role == PK_ROLE_ENUM_GET_DISTRO_UPGRADES) {
+			egg_debug ("cannot get lock for automatic action, ignoring");
+			goto out;
+		}
 	}
 
 	/* not modal as we are a status icon */
 	gpk_error_dialog (gpk_error_enum_to_localised_text (code),
 			  gpk_error_enum_to_localised_message (code), details);
+out:
+	return;
 }
 
 /**
@@ -1400,16 +1415,18 @@ gpk_check_update_finished_notify (GpkCheckUpdate *cupdate, PkClient *client)
 	}
 
 	/* TRANSLATORS: title: system update completed all okay */
-	notification = notify_notification_new (_("The system update has completed"), message_text->str, "help-browser", NULL);
+	notification = notify_notification_new_with_status_icon (_("The system update has completed"),
+								 message_text->str, "help-browser",
+								 cupdate->priv->status_icon);
 	notify_notification_set_timeout (notification, 15000);
 	notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
 	if (restart == PK_RESTART_ENUM_SYSTEM) {
 		notify_notification_add_action (notification, "restart",
 						/* TRANSLATORS: restart computer as system packages need update */
 						_("Restart computer now"), gpk_check_update_libnotify_cb, cupdate, NULL);
-		notify_notification_add_action (notification, "do-not-show-complete-restart",
-						/* TRANSLATORS: don't show this option again (for restart) */
-						_("Do not show this again"), gpk_check_update_libnotify_cb, cupdate, NULL);
+//		notify_notification_add_action (notification, "do-not-show-complete-restart",
+//						/* TRANSLATORS: don't show this option again (for restart) */
+//						_("Do not show this again"), gpk_check_update_libnotify_cb, cupdate, NULL);
 	} else {
 		notify_notification_add_action (notification, "do-not-show-complete",
 						/* TRANSLATORS: don't show this option again (when finished)  */
@@ -1509,6 +1526,7 @@ gpk_check_update_init (GpkCheckUpdate *cupdate)
 
 	cupdate->priv = GPK_CHECK_UPDATE_GET_PRIVATE (cupdate);
 
+	cupdate->priv->updates_changed_id = 0;
 	cupdate->priv->notification_updates_available = NULL;
 	cupdate->priv->important_updates_array = NULL;
 	cupdate->priv->icon_name = NULL;
@@ -1622,6 +1640,8 @@ gpk_check_update_finalize (GObject *object)
 		g_ptr_array_free (cupdate->priv->important_updates_array, TRUE);
 	}
 	g_free (cupdate->priv->icon_name);
+	if (cupdate->priv->updates_changed_id > 0)
+		g_source_remove (cupdate->priv->updates_changed_id);
 
 	G_OBJECT_CLASS (gpk_check_update_parent_class)->finalize (object);
 }
