@@ -24,7 +24,6 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
@@ -33,11 +32,10 @@
 #include <sys/types.h>
 #include <pwd.h>
 
-#include <polkit-gnome/polkit-gnome.h>
 #include <gconf/gconf-client.h>
 #include <packagekit-glib/packagekit.h>
+#include <unique/unique.h>
 
-#include "egg-unique.h"
 #include "egg-debug.h"
 #include "egg-string.h"
 
@@ -45,12 +43,11 @@
 #include "gpk-gnome.h"
 #include "gpk-enum.h"
 
-static GladeXML *glade_xml = NULL;
+static GtkBuilder *builder = NULL;
 static GtkListStore *list_store = NULL;
 static PkClient *client = NULL;
 static gchar *transaction_id = NULL;
 static gchar *filter = NULL;
-static PolKitGnomeAction *button_action = NULL;
 static PkObjList *transactions = NULL;
 static GtkTreePath *path_global = NULL;
 
@@ -159,24 +156,6 @@ gpk_log_button_help_cb (GtkWidget *widget, gpointer data)
 	gpk_gnome_help ("software-log");
 }
 
-/**
- * gpk_log_button_rollback_cb:
- **/
-static void
-gpk_log_button_rollback_cb (PolKitGnomeAction *action, gpointer data)
-{
-	gboolean ret;
-	GError *error = NULL;
-
-	/* rollback */
-	ret = pk_client_rollback (client, transaction_id, &error);
-	if (!ret) {
-		egg_warning ("failed to reset client: %s", error->message);
-		g_error_free (error);
-	}
-	gtk_main_quit ();
-}
-
 /**
  * gpk_log_get_localised_date:
  **/
@@ -392,50 +371,16 @@ gpk_log_treeview_clicked_cb (GtkTreeSelection *selection, gpointer data)
 }
 
 /**
- * gpk_update_viewer_create_custom_widget:
- **/
-static GtkWidget *
-gpk_update_viewer_create_custom_widget (GladeXML *xml, gchar *func_name, gchar *name,
-				        gchar *string1, gchar *string2,
-				        gint int1, gint int2, gpointer user_data)
-{
-	if (egg_strequal (name, "button_action"))
-		return polkit_gnome_action_create_button (button_action);
-	egg_warning ("name unknown=%s", name);
-	return NULL;
-}
-
-/**
- * gpk_update_viewer_setup_policykit:
- *
- * We have to do this before the glade stuff if done as the custom handler needs the actions setup
+ * gpk_log_message_received_cb
  **/
 static void
-gpk_update_viewer_setup_policykit (void)
+gpk_log_message_received_cb (UniqueApp *app, UniqueCommand command, UniqueMessageData *message_data, guint time_ms, gpointer data)
 {
-	PolKitAction *pk_action;
-	pk_action = polkit_action_new ();
-	polkit_action_set_action_id (pk_action, "org.freedesktop.packagekit.system-rollback");
-	/* TRANSLATORS: button label, roll back to a previous system snapshot */
-	button_action = polkit_gnome_action_new_default ("rollback", pk_action, _("_Rollback"), NULL);
-	g_object_set (button_action,
-		      "no-icon-name", "gtk-go-back-ltr",
-		      "auth-icon-name", "gtk-go-back-ltr",
-		      "yes-icon-name", "gtk-go-back-ltr",
-		      "self-blocked-icon-name", "gtk-go-back-ltr",
-		      NULL);
-	polkit_action_unref (pk_action);
-}
-
-/**
- * gpk_log_activated_cb
- **/
-static void
-gpk_log_activated_cb (EggUnique *egg_unique, gpointer data)
-{
-	GtkWidget *widget;
-	widget = glade_xml_get_widget (glade_xml, "dialog_simple");
-	gtk_window_present (GTK_WINDOW (widget));
+	GtkWindow *window;
+	if (command == UNIQUE_ACTIVATE) {
+		window = GTK_WINDOW (gtk_builder_get_object (builder, "dialog_simple"));
+		gtk_window_present (window);
+	}
 }
 
 /**
@@ -512,7 +457,7 @@ gpk_log_add_obj (const PkTransactionObj *obj)
 	const gchar *tool;
 	static guint count;
 	struct passwd *pw;
-	GtkTreeView *treeview = GTK_TREE_VIEW (glade_xml_get_widget (glade_xml, "treeview_simple"));
+	GtkTreeView *treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder, "treeview_simple"));
 	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
 
 	/* put formatted text into treeview */
@@ -533,19 +478,16 @@ gpk_log_add_obj (const PkTransactionObj *obj)
 	}
 
 	/* get nice name for tool name */
-	if (egg_strequal (obj->cmdline, "pkcon"))
+	if (g_strcmp0 (obj->cmdline, "pkcon") == 0)
 		/* TRANSLATORS: short name for pkcon */
 		tool = _("Command line client");
-	else if (egg_strequal (obj->cmdline, "gpk-application"))
+	else if (g_strcmp0 (obj->cmdline, "gpk-application") == 0)
 		/* TRANSLATORS: short name for gpk-update-viewer */
 		tool = _("Add/Remove Software");
-	else if (egg_strequal (obj->cmdline, "gpk-update-viewer"))
+	else if (g_strcmp0 (obj->cmdline, "gpk-update-viewer") == 0)
 		/* TRANSLATORS: short name for gpk-update-viewer */
 		tool = _("Update System");
-	else if (egg_strequal (obj->cmdline, "gpk-update-viewer2"))
-		/* TRANSLATORS: short name for gpk-update-viewer (new name) */
-		tool = _("Update System");
-	else if (egg_strequal (obj->cmdline, "gpk-update-icon"))
+	else if (g_strcmp0 (obj->cmdline, "gpk-update-icon") == 0)
 		/* TRANSLATORS: short name for gpk-update-icon */
 		tool = _("Update Icon");
 	else
@@ -589,7 +531,7 @@ gpk_log_refilter (void)
 
 	/* set the new filter */
 	g_free (filter);
-	widget = glade_xml_get_widget (glade_xml, "entry_package");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "entry_package"));
 	package = gtk_entry_get_text (GTK_ENTRY(widget));
 	if (!egg_strzero (package))
 		filter = g_strdup (package);
@@ -599,7 +541,7 @@ gpk_log_refilter (void)
 	egg_debug ("len=%i", transactions->len);
 
 	/* mark the items as not used */
-	treeview = GTK_TREE_VIEW (glade_xml_get_widget (glade_xml, "treeview_simple"));
+	treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder, "treeview_simple"));
 	model = gtk_tree_view_get_model (treeview);
 	gpk_log_mark_nonactive (model);
 
@@ -640,7 +582,11 @@ gpk_log_refresh (void)
 	/* get the list */
 	if (transactions != NULL)
 		g_object_unref (transactions);
-	transactions = pk_client_get_cached_objects (client);
+#if PK_CHECK_VERSION(0,5,0)
+	transactions = pk_client_get_transaction_list (client);
+#else
+	transactions = pk_client_get_cached_objects (client); /* removed in 0.5.x */
+#endif
 	gpk_log_refilter ();
 out:
 	return ret;
@@ -687,10 +633,11 @@ main (int argc, char *argv[])
 	GtkWidget *widget;
 	GtkTreeSelection *selection;
 	GtkEntryCompletion *completion;
-	PkBitfield roles;
-	PkControl *control;
-	EggUnique *egg_unique;
+	UniqueApp *unique_app;
 	gboolean ret;
+	guint retval;
+	guint xid = 0;
+	GError *error = NULL;
 
 	const GOptionEntry options[] = {
 		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
@@ -698,6 +645,9 @@ main (int argc, char *argv[])
 		{ "filter", 'f', 0, G_OPTION_ARG_STRING, &filter,
 		  /* TRANSLATORS: preset the GtktextBox with this filter text */
 		  N_("Set the filter to this value"), NULL },
+		{ "parent-window", 'p', 0, G_OPTION_ARG_INT, &xid,
+		  /* TRANSLATORS: we can make this modal (stay on top of) another window */
+		  _("Set the parent window to make this modal"), NULL },
 		{ NULL}
 	};
 
@@ -727,35 +677,33 @@ main (int argc, char *argv[])
 		return 1;
 
 	/* are we already activated? */
-	egg_unique = egg_unique_new ();
-	ret = egg_unique_assign (egg_unique, "org.freedesktop.PackageKit.LogViewer");
-	if (!ret)
+	unique_app = unique_app_new ("org.freedesktop.PackageKit.LogViewer", NULL);
+	if (unique_app_is_running (unique_app)) {
+		egg_debug ("You have another instance running. This program will now close");
+		unique_app_send_message (unique_app, UNIQUE_ACTIVATE, NULL);
 		goto unique_out;
-
-	g_signal_connect (egg_unique, "activated",
-			  G_CALLBACK (gpk_log_activated_cb), NULL);
+	}
+	g_signal_connect (unique_app, "message-received",
+			  G_CALLBACK (gpk_log_message_received_cb), NULL);
 
 	/* add application specific icons to search path */
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
 					   GPK_DATA G_DIR_SEPARATOR_S "icons");
 
-	/* we have to do this before we connect up the glade file */
-	gpk_update_viewer_setup_policykit ();
-
-	/* use custom widgets */
-	glade_set_custom_handler (gpk_update_viewer_create_custom_widget, NULL);
-
 	client = pk_client_new ();
 	pk_client_set_use_buffer (client, TRUE, NULL);
 	pk_client_set_synchronous (client, TRUE, NULL);
 
-	/* get actions */
-	control = pk_control_new ();
-	roles = pk_control_get_actions (control, NULL);
-	g_object_unref (control);
+	/* get UI */
+	builder = gtk_builder_new ();
+	retval = gtk_builder_add_from_file (builder, GPK_DATA "/gpk-log.ui", &error);
+	if (error != NULL) {
+		egg_warning ("failed to load ui: %s", error->message);
+		g_error_free (error);
+		goto out_build;
+	}
 
-	glade_xml = glade_xml_new (GPK_DATA "/gpk-log.glade", NULL, NULL);
-	widget = glade_xml_get_widget (glade_xml, "dialog_simple");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_simple"));
 	gtk_window_set_icon_name (GTK_WINDOW (widget), GPK_ICON_SOFTWARE_LOG);
 
 	/* set a size, if the screen allows */
@@ -763,26 +711,26 @@ main (int argc, char *argv[])
 
 	/* if command line arguments are set, then setup UI */
 	if (filter != NULL) {
-		widget = glade_xml_get_widget (glade_xml, "entry_package");
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "entry_package"));
 		gtk_entry_set_text (GTK_ENTRY(widget), filter);
 	}
 
 	/* Get the main window quit */
 	g_signal_connect_swapped (widget, "delete_event", G_CALLBACK (gtk_main_quit), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "button_close");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_close"));
 	g_signal_connect_swapped (widget, "clicked", G_CALLBACK (gtk_main_quit), NULL);
 	gtk_widget_grab_default (widget);
 
-	widget = glade_xml_get_widget (glade_xml, "button_help");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_help"));
 	g_signal_connect (widget, "clicked", G_CALLBACK (gpk_log_button_help_cb), NULL);
-	widget = glade_xml_get_widget (glade_xml, "button_refresh");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_refresh"));
 	g_signal_connect (widget, "clicked", G_CALLBACK (gpk_log_button_refresh_cb), NULL);
-	widget = glade_xml_get_widget (glade_xml, "button_filter");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_filter"));
 	g_signal_connect (widget, "clicked", G_CALLBACK (gpk_log_button_filter_cb), NULL);
 
 	/* hit enter in the search box for filter */
-	widget = glade_xml_get_widget (glade_xml, "entry_package");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "entry_package"));
 	g_signal_connect (widget, "activate", G_CALLBACK (gpk_log_button_filter_cb), NULL);
 
 	/* autocompletion can be turned off as it's slow */
@@ -791,31 +739,21 @@ main (int argc, char *argv[])
 	if (ret) {
 		/* create the completion object */
 		completion = gpk_package_entry_completion_new ();
-		widget = glade_xml_get_widget (glade_xml, "entry_package");
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "entry_package"));
 		gtk_entry_set_completion (GTK_ENTRY (widget), completion);
 		g_object_unref (completion);
 	} else {
 		/* use search as you type */
-		g_signal_connect (widget, "key-press-event", G_CALLBACK (gpk_log_entry_filter_cb), NULL);
 		g_signal_connect (widget, "key-release-event", G_CALLBACK (gpk_log_entry_filter_cb), NULL);
 	}
 	g_object_unref (gconf_client);
 
-	/* connect up PolicyKit actions */
-	g_signal_connect (button_action, "activate", G_CALLBACK (gpk_log_button_rollback_cb), NULL);
-
-	/* hide the rollback button if we can't do the action */
-	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_ROLLBACK))
-		polkit_gnome_action_set_visible (button_action, TRUE);
-	else
-		polkit_gnome_action_set_visible (button_action, FALSE);
-
 	/* create list stores */
 	list_store = gtk_list_store_new (GPK_LOG_COLUMN_LAST, G_TYPE_STRING, G_TYPE_STRING,
 					 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
 	/* create transaction_id tree view */
-	widget = glade_xml_get_widget (glade_xml, "treeview_simple");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_simple"));
 	gtk_tree_view_set_model (GTK_TREE_VIEW (widget),
 				 GTK_TREE_MODEL (list_store));
 
@@ -834,11 +772,19 @@ main (int argc, char *argv[])
 	gpk_log_refresh ();
 
 	/* show */
-	widget = glade_xml_get_widget (glade_xml, "dialog_simple");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_simple"));
 	gtk_widget_show (widget);
+
+	/* set the parent window if it is specified */
+	if (xid != 0) {
+		egg_debug ("Setting xid %i", xid);
+		gpk_window_set_parent_xid (GTK_WINDOW (widget), xid);
+	}
+
 	gtk_main ();
 
-	g_object_unref (glade_xml);
+out_build:
+	g_object_unref (builder);
 	g_object_unref (list_store);
 	g_object_unref (client);
 	g_free (transaction_id);
@@ -846,6 +792,6 @@ main (int argc, char *argv[])
 	if (transactions != NULL)
 		g_object_unref (transactions);
 unique_out:
-	g_object_unref (egg_unique);
+	g_object_unref (unique_app);
 	return 0;
 }
